"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ReactInspectorPlugin = void 0;
const middlewares_1 = require("./middlewares");
/**
 * [webpack service side]
 *
 * add launch editor middleware in webpack devServer.before method
 *
 * https://webpack.js.org/configuration/dev-server/#devserverbefore
 * https://webpack.js.org/contribute/writing-a-plugin
 */
class ReactInspectorPlugin {
    apply(compiler) {
        if (!compiler.options.devServer) {
            compiler.options.devServer = {};
        }
        const { devServer } = compiler.options;
        /**
         * for webpack@^5 + webpack-dev-server@^4.7
         */
        const originSetup = devServer.setupMiddlewares;
        devServer.setupMiddlewares = (middlewares, devServer) => {
            const result = originSetup ? originSetup(middlewares, devServer) : middlewares;
            result.unshift(middlewares_1.launchEditorMiddleware);
            return result;
        };
        /**
         * for webpack@^4 + webpack-dev-server@^3
         */
        const originBefore = devServer.before;
        devServer.before = (app, server, compiler) => {
            app.use(middlewares_1.launchEditorMiddleware);
            originBefore === null || originBefore === void 0 ? void 0 : originBefore(app, server, compiler);
        };
    }
}
exports.ReactInspectorPlugin = ReactInspectorPlugin;
