"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLaunchEditorMiddleware = exports.launchEditorMiddleware = exports.queryParserMiddleware = void 0;
const path_1 = __importDefault(require("path"));
/**
 * https://github.com/facebook/create-react-app/blob/v5.0.1/packages/react-dev-utils/launchEditorEndpoint.js
 * used in https://github.com/facebook/create-react-app/blob/v5.0.1/packages/react-dev-utils/errorOverlayMiddleware.js#L14
 */
const launchEditorEndpoint_1 = __importDefault(require("react-dev-utils/launchEditorEndpoint"));
const errorOverlayMiddleware_1 = __importDefault(require("react-dev-utils/errorOverlayMiddleware"));
const reactLaunchEditorMiddleware = (0, errorOverlayMiddleware_1.default)();
/**
 * @deprecated no longer necessary to use `queryParserMiddleware` directly,
 *   which has been integrated into the `launchEditorMiddleware`
 */
const queryParserMiddleware = (req, res, next) => {
    if (!req.query && req.url) {
        const url = new URL(req.url, 'https://placeholder.domain');
        req.query = Object.fromEntries(url.searchParams.entries());
    }
    next();
};
exports.queryParserMiddleware = queryParserMiddleware;
const launchEditorMiddleware = (req, res, next) => {
    var _a;
    if (!((_a = req.url) === null || _a === void 0 ? void 0 : _a.startsWith(launchEditorEndpoint_1.default))) {
        return next();
    }
    (0, exports.queryParserMiddleware)(req, res, () => { });
    if (!req.query.fileName) {
        return next();
    }
    if (!('REACT_EDITOR' in process.env)) {
        /**
         * If not set `REACT_EDITOR` in environment variables, set default to `vscode`.
         * - (`code` is cli command installed by `vscode`:  `>Shell Command: Install 'code' command in PATH`)
         * - link: https://code.visualstudio.com/docs/setup/mac#_launching-from-the-command-line
         *
         * ---
         *
         * `REACT_EDITOR` env var is used in `react-dev-utils/launchEditor`,
         * for launch editor(IDE) in backend of `REACT_EDITOR`.
         *
         * If not provide `REACT_EDITOR`, `react-dev-utils/launchEditor` will guess one editor(IDE) which user opened.
         *
         * They called "Auto-detect more common editors" -> (https://github.com/facebook/create-react-app/issues/2636)
         *
         * but sometimes the guess result is wrong, so we make it default to `vscode` for frontend developer.
         *
         * ---
         *
         * used in https://github.com/facebook/create-react-app/blob/v5.0.1/packages/react-dev-utils/launchEditor.js#L198
         */
        process.env.REACT_EDITOR = 'code';
    }
    /**
     * retain origin endpoint for backward compatibility <= v1.2.0
     */
    if (
    // relative route used in `Inspector.tsx` `gotoEditor()` relative path by
    // react-dev-inspector's babel plugin
    req.url.startsWith(`${launchEditorEndpoint_1.default}/relative`)
        && typeof req.query.fileName === 'string') {
        req.query.fileName = path_1.default.join(process.cwd(), req.query.fileName);
    }
    reactLaunchEditorMiddleware(req, res, next);
};
exports.launchEditorMiddleware = launchEditorMiddleware;
/**
 * retain create method for backward compatibility <= v1.2.0
 */
const createLaunchEditorMiddleware = () => exports.launchEditorMiddleware;
exports.createLaunchEditorMiddleware = createLaunchEditorMiddleware;
